"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Highlighter = void 0;
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var client_1 = tslib_1.__importDefault(require("react-dom/client"));
var index_1 = tslib_1.__importDefault(require("html-react-parser/lib/index"));
var serialize_1 = require("../../libs/serialize");
var uid_1 = require("../../libs/uid");
var wrapperElements_1 = require("../../libs/wrapperElements");
var DeafultPopover_1 = tslib_1.__importDefault(require("../DeafultPopover"));
var UseSelection_1 = require("../../hooks/UseSelection");
var constants_1 = require("../../constants/constants");
var dom_1 = require("../../libs/dom");
var Highlighter = function (_a) {
    var htmlString = _a.htmlString, onClickHighlight = _a.onClickHighlight, disablePopover = _a.disablePopover, maxSelectionLength = _a.maxSelectionLength, minSelectionLength = _a.minSelectionLength, className = _a.className, PopoverChildren = _a.PopoverChildren, PopoverClassName = _a.PopoverClassName, selectionWrapperClassName = _a.selectionWrapperClassName, onClick = _a.onClick;
    var _b = (0, UseSelection_1.useSelections)(), selections = _b.selections, addSelection = _b.addSelection, removeSelection = _b.removeSelection, updateSelection = _b.updateSelection;
    var popoverRoots = (0, react_1.useRef)({});
    var content = (0, react_1.useMemo)(function () { return (0, index_1.default)(htmlString); }, [htmlString]);
    var rootRef = (0, react_1.useRef)(null);
    var getWrapper = (0, react_1.useCallback)(function (selection) {
        var span = (0, wrapperElements_1.getSpanElement)({ className: selection.className || constants_1.defaultSelectionWrapperClassName });
        if (!disablePopover) {
            var popover_1 = (0, wrapperElements_1.getPopoverElement)({ className: PopoverClassName });
            if (!PopoverClassName) {
                span.onmouseover = function () {
                    popover_1.style.visibility = 'visible';
                    popover_1.style.opacity = '1';
                };
                span.onmouseout = function () {
                    popover_1.style.visibility = 'hidden';
                    popover_1.style.opacity = '0';
                };
            }
            popover_1.id = "pop-".concat(selection.id);
            span.appendChild(popover_1);
            var root = client_1.default.createRoot(popover_1);
            if (!popoverRoots.current[selection.id]) {
                popoverRoots.current[selection.id] = root;
            }
        }
        if (onClickHighlight) {
            span.onclick = function (e) { return onClickHighlight(selection, e); };
        }
        span.id = selection.id;
        return span;
    }, [PopoverClassName, disablePopover, onClickHighlight]);
    var handleMouseUp = function () {
        // e.stopPropagation()
        var selection = window.getSelection();
        if (!selection)
            return;
        if (!minSelectionLength) {
            minSelectionLength = constants_1.defaultMinSelectionLength;
        }
        if (minSelectionLength && selection.toString().length < minSelectionLength)
            return;
        if (maxSelectionLength && selection.toString().length > maxSelectionLength)
            return;
        var range = selection.getRangeAt(0);
        if (!(0, dom_1.isHighlightable)(range))
            return;
        var newSelection = {
            meta: (0, serialize_1.serializeRange)(range, rootRef.current),
            text: range.toString(),
            id: "selection-".concat((0, uid_1.generateId)()),
            className: selectionWrapperClassName || constants_1.defaultSelectionWrapperClassName,
            timestamp: Date.now(),
        };
        addSelection(newSelection);
    };
    (0, react_1.useEffect)(function () {
        var currentRefs = popoverRoots.current;
        var restore = function (selections) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var i, item, range;
            var _a, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        selections.sort(function (a, b) { return a.timestamp - b.timestamp; });
                        i = 0;
                        _c.label = 1;
                    case 1:
                        if (!(i < selections.length)) return [3 /*break*/, 4];
                        item = selections[i];
                        return [4 /*yield*/, (0, serialize_1.deserializeRange)(item.meta, rootRef.current)];
                    case 2:
                        range = _c.sent();
                        if (range) {
                            (0, dom_1.addHighlight)(range, getWrapper(item));
                        }
                        if (PopoverChildren) {
                            (_a = currentRefs[item.id]) === null || _a === void 0 ? void 0 : _a.render(react_1.default.createElement(PopoverChildren, { selection: item, removeSelection: removeSelection, updateSelection: updateSelection }));
                        }
                        else {
                            (_b = currentRefs[item.id]) === null || _b === void 0 ? void 0 : _b.render(react_1.default.createElement(DeafultPopover_1.default, { removeSelection: removeSelection, selection: item, updateSelection: updateSelection }));
                        }
                        _c.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        }); };
        if (selections && selections.length) {
            restore(selections);
        }
    }, [selections, getWrapper, PopoverChildren, removeSelection, updateSelection]);
    return (react_1.default.createElement("div", { ref: rootRef, id: 'highlighter-root', onClick: onClick, onMouseUp: handleMouseUp, className: className }, content));
};
exports.Highlighter = Highlighter;
