import { __awaiter, __generator } from "tslib";
import React, { useMemo, useRef, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom/client';
import HTMLReactParser from 'html-react-parser/lib/index';
import { deserializeRange, serializeRange } from '../../libs/serialize';
import { generateId } from '../../libs/uid';
import { getPopoverElement, getSpanElement } from '../../libs/wrapperElements';
import DefaultPopover from '../DeafultPopover';
import { useSelections } from '../../hooks/UseSelection';
import { defaultMinSelectionLength, defaultSelectionWrapperClassName } from '../../constants/constants';
import { addHighlight, isHighlightable } from '../../libs/dom';
export var Highlighter = function (_a) {
    var htmlString = _a.htmlString, onClickHighlight = _a.onClickHighlight, disablePopover = _a.disablePopover, maxSelectionLength = _a.maxSelectionLength, minSelectionLength = _a.minSelectionLength, className = _a.className, PopoverChildren = _a.PopoverChildren, PopoverClassName = _a.PopoverClassName, selectionWrapperClassName = _a.selectionWrapperClassName, onClick = _a.onClick;
    var _b = useSelections(), selections = _b.selections, addSelection = _b.addSelection, removeSelection = _b.removeSelection, updateSelection = _b.updateSelection;
    var popoverRoots = useRef({});
    var content = useMemo(function () { return HTMLReactParser(htmlString); }, [htmlString]);
    var rootRef = useRef(null);
    var getWrapper = useCallback(function (selection) {
        var span = getSpanElement({ className: selection.className || defaultSelectionWrapperClassName });
        if (!disablePopover) {
            var popover_1 = getPopoverElement({ className: PopoverClassName });
            if (!PopoverClassName) {
                span.onmouseover = function () {
                    popover_1.style.visibility = 'visible';
                    popover_1.style.opacity = '1';
                };
                span.onmouseout = function () {
                    popover_1.style.visibility = 'hidden';
                    popover_1.style.opacity = '0';
                };
            }
            popover_1.id = "pop-".concat(selection.id);
            span.appendChild(popover_1);
            var root = ReactDOM.createRoot(popover_1);
            if (!popoverRoots.current[selection.id]) {
                popoverRoots.current[selection.id] = root;
            }
        }
        if (onClickHighlight) {
            span.onclick = function (e) { return onClickHighlight(selection, e); };
        }
        span.id = selection.id;
        return span;
    }, [PopoverClassName, disablePopover, onClickHighlight]);
    var handleMouseUp = function () {
        // e.stopPropagation()
        var selection = window.getSelection();
        if (!selection)
            return;
        if (!minSelectionLength) {
            minSelectionLength = defaultMinSelectionLength;
        }
        if (minSelectionLength && selection.toString().length < minSelectionLength)
            return;
        if (maxSelectionLength && selection.toString().length > maxSelectionLength)
            return;
        var range = selection.getRangeAt(0);
        if (!isHighlightable(range))
            return;
        var newSelection = {
            meta: serializeRange(range, rootRef.current),
            text: range.toString(),
            id: "selection-".concat(generateId()),
            className: selectionWrapperClassName || defaultSelectionWrapperClassName,
            timestamp: Date.now(),
        };
        addSelection(newSelection);
    };
    useEffect(function () {
        var currentRefs = popoverRoots.current;
        var restore = function (selections) { return __awaiter(void 0, void 0, void 0, function () {
            var i, item, range;
            var _a, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        selections.sort(function (a, b) { return a.timestamp - b.timestamp; });
                        i = 0;
                        _c.label = 1;
                    case 1:
                        if (!(i < selections.length)) return [3 /*break*/, 4];
                        item = selections[i];
                        return [4 /*yield*/, deserializeRange(item.meta, rootRef.current)];
                    case 2:
                        range = _c.sent();
                        if (range) {
                            addHighlight(range, getWrapper(item));
                        }
                        if (PopoverChildren) {
                            (_a = currentRefs[item.id]) === null || _a === void 0 ? void 0 : _a.render(React.createElement(PopoverChildren, { selection: item, removeSelection: removeSelection, updateSelection: updateSelection }));
                        }
                        else {
                            (_b = currentRefs[item.id]) === null || _b === void 0 ? void 0 : _b.render(React.createElement(DefaultPopover, { removeSelection: removeSelection, selection: item, updateSelection: updateSelection }));
                        }
                        _c.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        }); };
        if (selections && selections.length) {
            restore(selections);
        }
    }, [selections, getWrapper, PopoverChildren, removeSelection, updateSelection]);
    return (React.createElement("div", { ref: rootRef, id: 'highlighter-root', onClick: onClick, onMouseUp: handleMouseUp, className: className }, content));
};
